require "import"
import "android.widget.*"
import "android.view.*"
import "android.util.TypedValue"
import "android.graphics.Typeface"
import "android.content.Context"
import "android.content.ClipData"
import "android.content.ClipboardManager"
import "cjson"

-- 数据存储键
local WHITE_LIST_KEY = "auto_speak_apps"
local BLACK_LIST_KEY = "auto_speak_blacklist_data"
local WHITE_KEYWORDS_KEY = "auto_speak_whitelist_data"

-- dp转px函数
local function dp2px(dp)
    local metrics = activity.getResources().getDisplayMetrics()
    return TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, dp, metrics)
end

-- 初始化数据
local whiteApps = luajava.astable(activity.getSharedData(WHITE_LIST_KEY) or HashSet())
local blackKeywords = cjson.decode(activity.getSharedData(BLACK_LIST_KEY) or "{}")
local whiteKeywords = cjson.decode(activity.getSharedData(WHITE_KEYWORDS_KEY) or "{}")

-- 应用列表
local apps = {
    "抖音", "抖音极速版", "抖音火山版", 
    "快手", "快手极速版", "微信视频号",
    "淘宝直播", "拼多多直播"
}

-- 预设黑名单关键词
local presetBlackKeywords = {
    "*进入直播间*", "*送出了*", "*来了*", 
    "*点赞*", "*关注了*", "*加入了*", 
    "*分享了*", "*购买了*", "*下单了*"
}

-- 主布局
local layout = LinearLayout(activity)
layout.setOrientation(LinearLayout.VERTICAL)
layout.setBackgroundColor(0xFFF5F5F5)
layout.setPadding(dp2px(16), dp2px(16), dp2px(16), dp2px(16))

-- 标题
local title = TextView(activity)
title.setTextSize(20)
title.setTextColor(0xFF333333)
title.setTypeface(Typeface.DEFAULT_BOLD)
title.setText("直播公屏朗读设置")
title.setPadding(0, 0, 0, dp2px(16))
layout.addView(title)

-- 创建下拉式面板函数（优化了无障碍提示）
local function createExpandablePanel(titleText, contentView)
    local panelLayout = LinearLayout(activity)
    panelLayout.setOrientation(LinearLayout.VERTICAL)
    panelLayout.setBackgroundResource(android.R.drawable.dialog_holo_light_frame)
    panelLayout.setPadding(0, 0, 0, dp2px(16))
    
    -- 标题栏
    local titleLayout = LinearLayout(activity)
    titleLayout.setOrientation(LinearLayout.HORIZONTAL)
    titleLayout.setPadding(dp2px(16), dp2px(12), dp2px(16), dp2px(12))
    titleLayout.setGravity(Gravity.CENTER_VERTICAL)
    titleLayout.setContentDescription(titleText.."，可展开，当前已折叠")
    
    local title = TextView(activity)
    title.setTextSize(16)
    title.setTextColor(0xFF333333)
    title.setText(titleText)
    title.setLayoutParams(LinearLayout.LayoutParams(0, LinearLayout.LayoutParams.WRAP_CONTENT, 1))
    
    local arrow = ImageView(activity)
    arrow.setImageResource(android.R.drawable.arrow_down_float)
    
    titleLayout.addView(title)
    titleLayout.addView(arrow)
    
    -- 内容区域（初始隐藏）
    contentView.setVisibility(View.GONE)
    contentView.setPadding(dp2px(16), dp2px(8), dp2px(16), dp2px(8))
    
    -- 点击事件（优化无障碍提示）
    local isExpanded = false
    titleLayout.onClick = function()
        if isExpanded then
            contentView.setVisibility(View.GONE)
            arrow.setImageResource(android.R.drawable.arrow_down_float)
            titleLayout.setContentDescription(titleText.."，可展开，当前已折叠")
            service.speak(titleText.."，已折叠")
        else
            contentView.setVisibility(View.VISIBLE)
            arrow.setImageResource(android.R.drawable.arrow_up_float)
            titleLayout.setContentDescription(titleText.."，可折叠，当前已展开")
            service.speak(titleText.."，已展开")
        end
        isExpanded = not isExpanded
    end
    
    -- 双击折叠/展开
    titleLayout.onLongClick = function()
        if isExpanded then
            contentView.setVisibility(View.GONE)
            arrow.setImageResource(android.R.drawable.arrow_down_float)
            titleLayout.setContentDescription(titleText.."，可展开，当前已折叠")
            service.speak(titleText.."，已折叠")
        else
            contentView.setVisibility(View.VISIBLE)
            arrow.setImageResource(android.R.drawable.arrow_up_float)
            titleLayout.setContentDescription(titleText.."，可折叠，当前已展开")
            service.speak(titleText.."，已展开")
        end
        isExpanded = not isExpanded
        return true
    end
    
    panelLayout.addView(titleLayout)
    panelLayout.addView(contentView)
    
    return panelLayout
end

-- 1. 应用选择面板
local appContent = LinearLayout(activity)
appContent.setOrientation(LinearLayout.VERTICAL)

local appCheckboxes = {}
for i, app in ipairs(apps) do
    local cb = CheckBox(activity)
    cb.setText(app)
    cb.setTextSize(14)
    cb.setTextColor(0xFF555555)
    cb.setChecked(table.find(whiteApps, app) ~= nil)
    appCheckboxes[app] = cb
    appContent.addView(cb)
end

local appPanel = createExpandablePanel("选择要朗读公屏的应用", appContent)
layout.addView(appPanel)

-- 2. 黑名单面板
local blacklistContent = LinearLayout(activity)
blacklistContent.setOrientation(LinearLayout.VERTICAL)

-- 预设黑名单
local presetTitle = TextView(activity)
presetTitle.setTextSize(14)
presetTitle.setTextColor(0xFF666666)
presetTitle.setText("预设黑名单:")
presetTitle.setPadding(0, dp2px(8), 0, dp2px(4))
blacklistContent.addView(presetTitle)

local blacklistCheckboxes = {}
for i, keyword in ipairs(presetBlackKeywords) do
    local cb = CheckBox(activity)
    cb.setText(keyword)
    cb.setTextSize(14)
    cb.setTextColor(0xFF555555)
    cb.setChecked(blackKeywords[keyword] == true)
    blacklistCheckboxes[keyword] = cb
    blacklistContent.addView(cb)
end

-- 自定义黑名单
local customTitle = TextView(activity)
customTitle.setTextSize(14)
customTitle.setTextColor(0xFF666666)
customTitle.setText("自定义黑名单:")
customTitle.setPadding(0, dp2px(12), 0, dp2px(4))
blacklistContent.addView(customTitle)

local customLayout = LinearLayout(activity)
customLayout.setOrientation(LinearLayout.HORIZONTAL)

local customInput = EditText(activity)
customInput.setHint("输入关键词")
customInput.setBackgroundResource(android.R.drawable.edit_text)
customInput.setPadding(dp2px(8), dp2px(8), dp2px(8), dp2px(8))
customInput.setLayoutParams(LinearLayout.LayoutParams(0, dp2px(36), 1))

local addBtn = Button(activity)
addBtn.setText("添加")
addBtn.setTextSize(14)
addBtn.setBackgroundResource(android.R.drawable.btn_default)
addBtn.setMinWidth(dp2px(64))
addBtn.setLayoutParams(LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, dp2px(36)))
addBtn.onClick = function()
    local text = customInput.getText().toString()
    if text and #text > 0 then
        blackKeywords[text] = true
        local cb = CheckBox(activity)
        cb.setText(text)
        cb.setTextSize(14)
        cb.setTextColor(0xFF555555)
        cb.setChecked(true)
        blacklistCheckboxes[text] = cb
        blacklistContent.addView(cb, blacklistContent.getChildCount()-2)
        customInput.setText("")
        service.speak("已添加黑名单关键词："..text)
    end
end

customLayout.addView(customInput)
customLayout.addView(addBtn)
blacklistContent.addView(customLayout)

local blacklistPanel = createExpandablePanel("黑名单设置（不朗读）", blacklistContent)
layout.addView(blacklistPanel)

-- 3. 白名单面板
local whitelistContent = LinearLayout(activity)
whitelistContent.setOrientation(LinearLayout.VERTICAL)

local whitelistTitle = TextView(activity)
whitelistTitle.setTextSize(14)
whitelistTitle.setTextColor(0xFF666666)
whitelistTitle.setText("白名单关键词（优先朗读）:")
whitelistTitle.setPadding(0, dp2px(8), 0, dp2px(4))
whitelistContent.addView(whitelistTitle)

local whitelistLayout = LinearLayout(activity)
whitelistLayout.setOrientation(LinearLayout.HORIZONTAL)

local whitelistInput = EditText(activity)
whitelistInput.setHint("输入关键词，多个用逗号分隔")
whitelistInput.setBackgroundResource(android.R.drawable.edit_text)
whitelistInput.setPadding(dp2px(8), dp2px(8), dp2px(8), dp2px(8))
whitelistInput.setLayoutParams(LinearLayout.LayoutParams(0, dp2px(36), 1))

local addWhiteBtn = Button(activity)
addWhiteBtn.setText("添加")
addWhiteBtn.setTextSize(14)
addWhiteBtn.setBackgroundResource(android.R.drawable.btn_default)
addWhiteBtn.setMinWidth(dp2px(64))
addWhiteBtn.setLayoutParams(LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, dp2px(36)))
addWhiteBtn.onClick = function()
    local text = whitelistInput.getText().toString()
    if text and #text > 0 then
        for word in text:gmatch("[^,]+") do
            whiteKeywords[word] = true
            Toast.makeText(activity, "已添加: "..word, Toast.LENGTH_SHORT).show()
            service.speak("已添加白名单关键词："..word)
        end
        whitelistInput.setText("")
    end
end

whitelistLayout.addView(whitelistInput)
whitelistLayout.addView(addWhiteBtn)
whitelistContent.addView(whitelistLayout)

local whitelistPanel = createExpandablePanel("白名单设置（优先朗读）", whitelistContent)
layout.addView(whitelistPanel)

-- 4. 关于面板
local aboutContent = LinearLayout(activity)
aboutContent.setOrientation(LinearLayout.VERTICAL)
aboutContent.setPadding(dp2px(16), dp2px(8), dp2px(16), dp2px(8))

local authorTitle = TextView(activity)
authorTitle.setTextSize(16)
authorTitle.setTextColor(0xFF333333)
authorTitle.setText("作者：白茶不语")
authorTitle.setPadding(0, dp2px(8), 0, dp2px(8))
aboutContent.addView(authorTitle)

local qqLayout = LinearLayout(activity)
qqLayout.setOrientation(LinearLayout.HORIZONTAL)
qqLayout.setGravity(Gravity.CENTER_VERTICAL)

local qqText = TextView(activity)
qqText.setTextSize(16)
qqText.setTextColor(0xFF333333)
qqText.setText("QQ：1670675973")
qqText.setPadding(0, dp2px(8), dp2px(16), dp2px(8))
qqLayout.addView(qqText)

local copyBtn = Button(activity)
copyBtn.setText("一键复制")
copyBtn.setTextSize(14)
copyBtn.setBackgroundResource(android.R.drawable.btn_default)
copyBtn.setMinWidth(dp2px(80))
copyBtn.setLayoutParams(LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, dp2px(36)))
copyBtn.onClick = function()
    local clipboard = activity.getSystemService(Context.CLIPBOARD_SERVICE)
    local clip = ClipData.newPlainText("QQ号", "1670675973")
    clipboard.setPrimaryClip(clip)
    Toast.makeText(activity, "已复制QQ号", Toast.LENGTH_SHORT).show()
    service.speak("已复制QQ号1670675973")
end
qqLayout.addView(copyBtn)

aboutContent.addView(qqLayout)

local aboutPanel = createExpandablePanel("关于", aboutContent)
layout.addView(aboutPanel)

-- 保存按钮
local saveBtn = Button(activity)
saveBtn.setText("保存设置")
saveBtn.setTextSize(16)
saveBtn.setTextColor(0xFFFFFFFF)
saveBtn.setBackgroundResource(android.R.drawable.btn_default)
saveBtn.setBackgroundColor(0xFF4CAF50)
saveBtn.setLayoutParams(LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, dp2px(48)))
saveBtn.onClick = function()
    -- 保存应用白名单
    local newWhiteApps = {}
    for app, cb in pairs(appCheckboxes) do
        if cb.isChecked() then
            table.insert(newWhiteApps, app)
        end
    end
    activity.setSharedData(WHITE_LIST_KEY, HashSet(ArrayList(newWhiteApps)))
    
    -- 保存黑名单
    local newBlackKeywords = {}
    for keyword, cb in pairs(blacklistCheckboxes) do
        if cb.isChecked() then
            newBlackKeywords[keyword] = true
        end
    end
    activity.setSharedData(BLACK_LIST_KEY, cjson.encode(newBlackKeywords))
    
    -- 保存白名单
    activity.setSharedData(WHITE_KEYWORDS_KEY, cjson.encode(whiteKeywords))
    
    Toast.makeText(activity, "设置已保存", Toast.LENGTH_SHORT).show()
    service.speak("设置已保存")
end

layout.addView(saveBtn)

-- 设置主视图
activity.setContentView(layout)
